#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"

// Conns
#define TFT_DC 9
#define TFT_CS 10
#define TFT_RST 8
#define TFT_MISO 12
#define TFT_MOSI 11
#define TFT_CLK 13

Adafruit_ILI9341 lcd = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_MOSI, TFT_CLK, TFT_RST, TFT_MISO); //TFT Screen Connection

// Colors
#define BLACK_PAWN_COLOR 0x019F
#define WHITE_PAWN_COLOR 0xF972
#define WHITE 0xEF5D
#define BLACK ILI9341_BLACK
#define GREEN 0x6FEB
#define YELLOW ILI9341_YELLOW

// Activation States
#define A1 B100100
#define A2 B100010
#define A3 B100001
#define B1 B010100
#define B2 B010010
#define B3 B010001
#define C1 B001100
#define C2 B001010
#define C3 B001001
#define NO_TOUCH B000000
#define ALL_TOUCH B111111
#define C_TOUCH B2 // Center touch

// Pawn Colors For State
#define B_PAWN 0
#define W_PAWN 1
#define N_PAWN -1

// Timing 
#define NATURAL_RESPONSE 4000

// Init Dimensions
int width = lcd.width();
int height = lcd.height();

int max_dim = max(width, height);
int min_dim = min(width, height);
int padding = (max_dim - min_dim) / 2;

int section = width / 3;
int halfLenSide = (section - 20) / 2;
int lenSide = halfLenSide * 2;

// Square Centers
int x0c = 11 + halfLenSide;
int x1c = 11 + halfLenSide + section;
int x2c = 11 + halfLenSide + section * 2;

int y0c = padding + 10 + halfLenSide;
int y1c = padding + 10 + halfLenSide + section;
int y2c = padding + 10 + halfLenSide + section * 2;

// States
int touched_state = 0b100100; //  000 Row, 000 Columns, 00 empty (1 byte)
int turn_state = 1; // 1 = you, 0 = avr - human always starts
int game_count = 0; // Number of games played
int turn_number = 1; // Turn number
int turn_number_black = 0; //for move set index

int board_state[9][5] = { // <Pawn present (0:no, 1:yes), Pawn color (0: black, 1:white, -1:none), xCenter, yCenter, touched_state (0:untouched, 1:touched)>
  {1, B_PAWN, x0c, y0c, 0}, {1, B_PAWN, x1c, y0c, 0}, {1, B_PAWN, x2c, y0c, 0},
  {0, N_PAWN, x0c, y1c, 0}, {0, N_PAWN, x1c, y1c, 0}, {0, N_PAWN, x2c, y1c, 0},
  {1, W_PAWN, x0c, y2c, 0}, {1, W_PAWN, x1c, y2c, 0}, {1, W_PAWN, x2c, y2c, 0}
};

int activation_states[9] = {
  A1, A2, A3, B1, B2, B3, C1, C2, C3
};

int moves[10] = {0,0,0,0,0,0,0,0,0,0};

int counter_of_moves = 0;

//int blackList[][10] = {};// {moves, turn}

boolean gameFinished = false;


void setup() {
  lcd.begin();
  lcd.fillScreen(BLACK); // clear screen
}


void loop(void) {
  // ** TEST ** //
  //touched_state = NO_TOUCH;
  // touchStateUpdater();
  // delay(500);
  // touched_state = ALL_TOUCH;
  // touchStateUpdater();
  // delay(500);
  // ** ** ** //
  
  drawChessArea();
  delay(NATURAL_RESPONSE/2);
  touched_state = C_TOUCH; // Simulated touch
  touchStateUpdater();
  delay(NATURAL_RESPONSE/2);
  if (touched_state == C_TOUCH) { // center press -> game starts
    drawFillCenteredSquare(x1c, y1c, lenSide + 12, BLACK);
    while (gameFinished == false){
      playTurn();
    }
  }
}

boolean lock_resources(int pad){
  int pawn_color = board_state[pad][1];
  int activation = board_state[pad][4];   

  String turnText = "";
  String command = "";
  
  if(pawn_color == B_PAWN && turn_state != 0) { // Black Pawn
    if (activation == 1) {
      turnText = "That's My Pawn!";
      command = "Human Moves, Not AVR.";
      header_write(turnText, command);
      
      while (activation == 1){
        touchStateUpdater();
        activation = board_state[pad][4];
        
        // ** TEST ** //
        //delay(1000);
        //touched_state = NO_TOUCH;
        // ** --- ** //
      } // LOCKED     
      return false;
    }
  }

  else if(pawn_color == N_PAWN) { // No Pawn
    if (activation == 1) {
      // Text
      lcd.setTextColor(WHITE);
      turnText = "In The End I Will Always Win...";
      command = "Press For 1 Second To Reset";
      header_write(turnText, command);
      
      while (activation == 1){
                                                                              // ** TODO: reset ** //
        touchStateUpdater();
        activation = board_state[pad][4];
        
        // ** TEST ** //
        //delay(1000);
        //touched_state = NO_TOUCH;
        // ** --- ** //
      } // LOCKED     
      return false;
    }
  }


  else if(pawn_color == W_PAWN && turn_state == 1) { // WHITE Pawn
    if (activation == 1) {
      boolean possible_move = true;
      while(possible_move){
        while (activation == 1){
          touchStateUpdater();
          activation = board_state[pad][4];
          
          // ** TEST ** //
          delay(NATURAL_RESPONSE/2);
          touched_state = B3;
          // ** --- ** //
        } // LOCKED 
        
        for (int i = 0; i < 9; i++) {
           if(board_state[i][4] == 1){
              int new_active_xc = board_state[i][2];
              int new_active_yc = board_state[i][3];
              
              int old_active_xc = board_state[pad][2];
              int old_active_yc = board_state[pad][3];

              int diff_xc = old_active_xc - new_active_xc; // one way up
              //int diff_xc = new_active_xc - old_active_xc; // one way up // Forceful wrong direction
              int diff_yc = abs(old_active_yc - new_active_yc); // one way up

              if (board_state[i][1] == W_PAWN){
                  return false; 
              }
                
              if(diff_xc < 0){
                turnText = "Nice Try ;p";
                command = "That's An Illigal Move, Wrong Direction";
                header_write(turnText, command);
                delay(NATURAL_RESPONSE);
                return false;
              }
              
              if(diff_xc == 160 || diff_yc == 160){
                turnText = "Pawns Don't Jump!";
                command = "That's An Illigal Move, Too Far";
                header_write(turnText, command);
                delay(NATURAL_RESPONSE);
                return false;
              }

              if(diff_xc == 0 && diff_yc == 80){
                if (board_state[i][1] == B_PAWN){
                  turnText = "You Shal Not Pass";
                  command = "Can only move forwards if sapace is free";
                  header_write(turnText, command);
                  delay(NATURAL_RESPONSE);
                  return false; 
                }
                if (board_state[i][1] == N_PAWN){
                  board_state[pad][0] = 0; // {1, B_PAWN, x0c, y0c, 0}
                  board_state[pad][1] = N_PAWN;
                    
                  board_state[i][0] = 1; // {1, B_PAWN, x0c, y0c, 0}
                  board_state[i][1] = W_PAWN;
                  draw_pawns();
                  
                  turnText = "I See... Let Me Think!";
                  command = "You Moved One Up";
                  header_write(turnText, command);
  
                  delay(NATURAL_RESPONSE);
                  return true;
                }
              }

              if(diff_xc == 80 && diff_yc == 80){
                if (board_state[i][1] == B_PAWN){
                  turnText = "Ouch!";
                  command = "You Moved One Diagonally & Ate";
                  header_write(turnText, command);
                  pause();
              
                  delay(NATURAL_RESPONSE);
                  return true; 
                }
                if (board_state[i][1] == N_PAWN){
                  turnText = "Hahahah! You Missed";
                  command = "You Can Only Move Diagonally To Eat";
                  header_write(turnText, command);
                  delay(NATURAL_RESPONSE);
                  return false; 
                }
                if (board_state[i][1] == W_PAWN){
                  turnText = "Thanks For Trying To Help :D";
                  command = "Cannot Eat Your Own Pawns";
                  header_write(turnText, command);
                  delay(NATURAL_RESPONSE);
                  return false; 
                }
              }
              
              lcd.setCursor(width / 2, height / 2);
              lcd.print(diff_xc);
           }
        }
      }
    } 
  }
}

void pause(){
  while(1){}
}

void header_clear(){
    int start_x = 10;  
    lcd.fillRect(start_x+30, 0, width-30, padding/2, BLACK);
    lcd.fillRect(start_x, padding/2, width, padding/2, BLACK);
}

void header_write(String turnText, String command){                                                           
    header_clear();
    
    // Text
    lcd.setTextColor(WHITE);
    lcd.setCursor(width / 2 - turnText.length() * 3, 10);
    lcd.print(turnText);
    
    // Command
    lcd.setTextColor(GREEN);
    lcd.setCursor(width / 2 - command.length() * 3, 20);
    lcd.print(command);
}

void playTurn() { // ---------------------------------------------------------------------------> playTurn
  lcd.fillRect(0, 0, width, padding, BLACK); // Clear Header Text
    
  lcd.setTextColor(WHITE);
  String numTurnText= "T" + (String) turn_number;
  lcd.setCursor(5, 5);
  lcd.print(numTurnText);

  String turnText = "";
  String command = "";
   
  if (turn_state == 1) { // Your turn
    turnText = "Show Me What You Got!";
    command = "Please Make A Move";
    header_write(turnText, command);
    
    boolean has_made_move = false; 
    // ** TEST ** //
    touched_state = C3;
    // ** ** ** //
    while(has_made_move == false) {
      for (int i = 0; i < 9; i++) {
        touchStateUpdater();
        has_made_move = lock_resources(i);
      }
    }
    turn_state = 0;
  }

  else if (turn_state == 0) {
    turnText = "Good move, let me retort ;)";
    command = "Please Move Pawn As Shown Below";  
    header_write(turnText, command);

    checkGameState();

    int choice = 0;
    int rdm = 0;
    while(choice == 0){
      rdm = random(0, 3);
      choice = moves[rdm]; 
    }
    header_write((String)choice, ":)");
    int dest = 0;
    int i = 1;
    while(i > 0) {
        dest = choice%10;
        choice /= 10;
        --i;
    }
    
    choice = moves[rdm]; 
    int source = 0;
    i = 2;
    while(i >= 0) {
        source = choice%10;
        choice /= 10;
        --i;
    }

    x0c = board_state[source][2];
    y0c = board_state[source][3];
    
    x1c = board_state[dest][2];
    y1c = board_state[dest][3];
    
    drawCrossHair(x0c, y0c, x1c, y1c);
    
    turn_state = 1;
    turn_number_black += 1;
  }
  turn_number += 1;
}

void touchStateUpdater() {
  for (int i; i < 9; i++) {
    int xc = board_state[i][2];
    int yc = board_state[i][3];
    
    int on = 1;
    int off = 0;
    
    if (activation_states[i] == (activation_states[i] & touched_state)) {
      board_state[i][4] = on;
      drawCenteredSquare(xc, yc, lenSide+2, YELLOW);
    }
    else {
      board_state[i][4] = off;
      drawCenteredSquare(xc, yc, lenSide+2, BLACK);
    }
  }
}

void draw_pawns() {
  for (int i = 0; i < 9; i++) {
    int xc = board_state[i][2];
    int yc = board_state[i][3];
    
    if (board_state[i][1] == B_PAWN) {
      lcd.fillCircle(xc, yc, 25, BLACK_PAWN_COLOR);
    }
    else if (board_state[i][1] == W_PAWN) {
      lcd.fillCircle(xc, yc, 25, WHITE_PAWN_COLOR);
    }
    else if (board_state[i][1] == N_PAWN) {
       lcd.fillCircle(xc, yc, 25, BLACK);
    }
  }
}

void draw_board() {
  lcd.fillRect(2, padding + 1, min_dim - 4, min_dim - 4, BLACK);
    
  // Vertical
  lcd.drawLine(width / 3, padding + 1, width / 3, height - padding - 3, WHITE);
  lcd.drawLine((width / 3) * 2, padding + 1, (width / 3) * 2, height - padding - 3, WHITE);

  // Horizontal
  lcd.drawLine(2, width / 3 + padding, width - 1, width / 3 + padding, WHITE);
  lcd.drawLine(2, (width / 3) * 2 + padding, width - 1, (width / 3) * 2 + padding, WHITE);
  
  draw_pawns();
}

void drawChessArea() {
  // Outline Play Area
  lcd.drawRect(1, padding, min_dim - 1, min_dim - 1, WHITE);
  
  // Draw Pawns
  draw_board();
      
  // Greeting
  String greeting = "Let's Play!";
  String command = "Touch center pad to start playing :)";
  header_write(greeting, command);

  // Games Played
  String game_count_text = "Number of Games Played";
  lcd.setCursor(width - game_count_text.length() * 6 - 6, height - 30);
  lcd.print(game_count_text);
  lcd.setCursor(width - game_count_text.length() * 3 - 6, height - 15);
  lcd.print(game_count);
  lcd.drawRect(1, height - 37, 97, 37, WHITE);

  // Legend
  String legend_one = "- Black (AVR)";
  lcd.setCursor(17, height - 30);
  lcd.print(legend_one);
  lcd.fillCircle(8, height - 28, 5, BLACK_PAWN_COLOR);

  String legend_two = "- White (You)";
  lcd.setCursor(17, height - 15);
  lcd.print(legend_two);
  lcd.fillCircle(8, height - 13, 5, WHITE_PAWN_COLOR);

  // Cross Hair
  drawCrossHair(x1c, y1c, x1c, y1c);
  String pressMe = "**PRESS ME**";
  lcd.setCursor(x1c - pressMe.length() * 3, y1c + 15);
  lcd.setTextColor(GREEN);
  lcd.print(pressMe);
}

void drawCenteredSquare(int xc, int yc, int lenSide, uint16_t color)
{
  lcd.drawRect(xc - halfLenSide, yc - halfLenSide, lenSide, lenSide, color);
}

void drawFillCenteredSquare(int xc, int yc, int lenSide, uint16_t color)
{
  lcd.fillRect(xc - lenSide / 2, yc - lenSide / 2, lenSide, lenSide, color);
}

void drawCrossHair(int xSource, int ySource, int x, int y)
{
  int lenSize = 15;
  int x0 = x - (lenSize / 2);
  int y0 = y - (lenSize / 2);

  int x1 = x + (lenSize / 2);
  int y1 = y + (lenSize / 2);

  int xMid = (x1 + x0) / 2;
  int yMid = (y1 + y0) / 2;

  lcd.drawRect(x0, y0, lenSize, lenSize, GREEN);

  lcd.drawLine(x0, y0, x1, y1, GREEN);
  lcd.drawLine(x1, y0, x0, y1, GREEN);

  lcd.drawLine(xSource, ySource, xMid, yMid, GREEN); // 0x6FEB
}

void init_moves(){
  for (int j = 0; j < 4; j++) {
    moves[j] = 0;
  }
}

String getWhiteID(){
  String id = "";
    for (int i = 0; i < 9; i++) {
      if(board_state[i][1] == W_PAWN){
        id += (String)i;
      }
    }
   return id;
}

void checkGameState(){
  counter_of_moves = 0;
    for (int i = 0; i < 9; i++) {
      if(board_state[i][1] == B_PAWN){
        for (int j = 0; j < 9; j++) {
          int new_active_xc = board_state[j][2];
          int new_active_yc = board_state[j][3];
          
          int old_active_xc = board_state[i][2];
          int old_active_yc = board_state[i][3];
          
          int diff_xc = new_active_xc - old_active_xc; // one way up
          int diff_yc = abs(old_active_yc - new_active_yc); // one way up
          
          if(diff_xc == 0 && diff_yc == 80){
            if (board_state[j][1] == N_PAWN){
              moves[counter_of_moves] = i*10 + j;
              counter_of_moves += 1;
            }
          }
          
          if(diff_xc == 80 && diff_yc == 80){
            if (board_state[j][1] == W_PAWN){
              moves[counter_of_moves] = i*10+ j;
              counter_of_moves += 1;
            }
          }
        }
      }
    } 
}
